/*
 * Copyright Contributors to the Eclipse BlueChi project
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 */
#pragma once

#include <stdint.h>
#include <systemd/sd-bus.h>

#include "libbluechi/common/common.h"
#include "libbluechi/socket.h"

/* return < 0 for error, 0 to continue, 1 to stop, 2 to continue and skip (if value was not consumed) */
typedef int (*bus_property_cb)(const char *key, const char *value_type, sd_bus_message *m, void *userdata);

int bus_parse_properties_foreach(sd_bus_message *m, bus_property_cb cb, void *userdata);

int bus_parse_property_string(sd_bus_message *m, const char *name, const char **value);
char *bus_path_escape(const char *str);

typedef struct UnitInfo UnitInfo;
struct UnitInfo {
        int ref_count;

        char *node;
        char *id;
        char *description;
        char *load_state;
        char *active_state;
        char *sub_state;
        char *following;
        char *unit_path;
        uint32_t job_id;
        char *job_type;
        char *job_path;

        LIST_FIELDS(UnitInfo, units);
};

UnitInfo *new_unit();
UnitInfo *unit_ref(UnitInfo *unit);
void unit_unref(UnitInfo *unit);

int bus_parse_unit_info(sd_bus_message *message, UnitInfo *u);

int bus_socket_set_options(sd_bus *bus, SocketOptions *opts);

bool bus_id_is_valid(const char *name);

int assemble_object_path_string(const char *prefix, const char *name, char **res);

DEFINE_CLEANUP_FUNC(UnitInfo, unit_unref)
#define _cleanup_unit_ _cleanup_(unit_unrefp)

typedef struct UnitFileInfo UnitFileInfo;
struct UnitFileInfo {
        int ref_count;

        char *node;
        char *unit_path;
        char *enablement_status;

        LIST_FIELDS(UnitFileInfo, unit_files);
};

UnitFileInfo *new_unit_file();
UnitFileInfo *unit_file_ref(UnitFileInfo *unit_file);
void unit_file_unref(UnitFileInfo *unit_file);

int bus_parse_unit_file_info(sd_bus_message *m, UnitFileInfo *unit_file);

DEFINE_CLEANUP_FUNC(UnitFileInfo, unit_file_unref)
#define _cleanup_unit_file_ _cleanup_(unit_file_unrefp)
